# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './snitch/ui/diff.ui'
#
# Created by: PyQt5 UI code generator 5.14.1
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Diff(object):
    def setupUi(self, Diff):
        Diff.setObjectName("Diff")
        Diff.resize(1094, 681)
        Diff.setAutoFillBackground(True)
        self.gridLayout = QtWidgets.QGridLayout(Diff)
        self.gridLayout.setObjectName("gridLayout")
        self.widget = QtWidgets.QWidget(Diff)
        self.widget.setObjectName("widget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.widget)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(646, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.highlightButton = QtWidgets.QPushButton(self.widget)
        self.highlightButton.setCheckable(True)
        self.highlightButton.setObjectName("highlightButton")
        self.horizontalLayout.addWidget(self.highlightButton)
        self.pushButton = QtWidgets.QPushButton(self.widget)
        self.pushButton.setObjectName("pushButton")
        self.horizontalLayout.addWidget(self.pushButton)
        self.gridLayout.addWidget(self.widget, 0, 0, 1, 2)
        self.referenceGroupBox = QtWidgets.QGroupBox(Diff)
        self.referenceGroupBox.setObjectName("referenceGroupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.referenceGroupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.referenceScrollArea = QtWidgets.QScrollArea(self.referenceGroupBox)
        self.referenceScrollArea.setWidgetResizable(True)
        self.referenceScrollArea.setObjectName("referenceScrollArea")
        self.referenceScrollAreaContents = QtWidgets.QWidget()
        self.referenceScrollAreaContents.setGeometry(QtCore.QRect(0, 0, 507, 466))
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(51, 51, 51))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Window, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(51, 51, 51))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Window, brush)
        brush = QtGui.QBrush(QtGui.QColor(51, 51, 51))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(51, 51, 51))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Window, brush)
        self.referenceScrollAreaContents.setPalette(palette)
        self.referenceScrollAreaContents.setObjectName("referenceScrollAreaContents")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.referenceScrollAreaContents)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.referenceImageLabel = QtWidgets.QLabel(self.referenceScrollAreaContents)
        self.referenceImageLabel.setText("")
        self.referenceImageLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.referenceImageLabel.setObjectName("referenceImageLabel")
        self.horizontalLayout_2.addWidget(self.referenceImageLabel)
        self.referenceScrollArea.setWidget(self.referenceScrollAreaContents)
        self.verticalLayout.addWidget(self.referenceScrollArea)
        self.referenceTextEdit = QtWidgets.QPlainTextEdit(self.referenceGroupBox)
        self.referenceTextEdit.setMaximumSize(QtCore.QSize(16777215, 120))
        font = QtGui.QFont()
        font.setFamily("Monospace")
        self.referenceTextEdit.setFont(font)
        self.referenceTextEdit.setUndoRedoEnabled(False)
        self.referenceTextEdit.setReadOnly(True)
        self.referenceTextEdit.setObjectName("referenceTextEdit")
        self.verticalLayout.addWidget(self.referenceTextEdit)
        self.gridLayout.addWidget(self.referenceGroupBox, 1, 0, 1, 1)
        self.groupBox = QtWidgets.QGroupBox(Diff)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.resultScrollArea = QtWidgets.QScrollArea(self.groupBox)
        self.resultScrollArea.setWidgetResizable(True)
        self.resultScrollArea.setObjectName("resultScrollArea")
        self.resultScrollAreaContents = QtWidgets.QWidget()
        self.resultScrollAreaContents.setGeometry(QtCore.QRect(0, 0, 507, 466))
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(51, 51, 51))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Window, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(51, 51, 51))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Window, brush)
        brush = QtGui.QBrush(QtGui.QColor(51, 51, 51))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(51, 51, 51))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Window, brush)
        self.resultScrollAreaContents.setPalette(palette)
        self.resultScrollAreaContents.setObjectName("resultScrollAreaContents")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.resultScrollAreaContents)
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_3.setSpacing(0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.resultImageLabel = QtWidgets.QLabel(self.resultScrollAreaContents)
        self.resultImageLabel.setText("")
        self.resultImageLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.resultImageLabel.setObjectName("resultImageLabel")
        self.horizontalLayout_3.addWidget(self.resultImageLabel)
        self.resultScrollArea.setWidget(self.resultScrollAreaContents)
        self.verticalLayout_2.addWidget(self.resultScrollArea)
        self.resultTextEdit = QtWidgets.QPlainTextEdit(self.groupBox)
        self.resultTextEdit.setMaximumSize(QtCore.QSize(16777215, 120))
        font = QtGui.QFont()
        font.setFamily("Monospace")
        self.resultTextEdit.setFont(font)
        self.resultTextEdit.setUndoRedoEnabled(False)
        self.resultTextEdit.setReadOnly(True)
        self.resultTextEdit.setObjectName("resultTextEdit")
        self.verticalLayout_2.addWidget(self.resultTextEdit)
        self.gridLayout.addWidget(self.groupBox, 1, 1, 1, 1)

        self.retranslateUi(Diff)
        self.highlightButton.released.connect(Diff.swap_results)
        self.pushButton.released.connect(Diff.save_images)
        QtCore.QMetaObject.connectSlotsByName(Diff)

    def retranslateUi(self, Diff):
        _translate = QtCore.QCoreApplication.translate
        Diff.setWindowTitle(_translate("Diff", "Screenshot comparison"))
        self.highlightButton.setText(_translate("Diff", "Highlight differences"))
        self.pushButton.setText(_translate("Diff", "Export as image…"))
        self.referenceGroupBox.setTitle(_translate("Diff", "Reference"))
        self.groupBox.setTitle(_translate("Diff", "Test result"))
